/*
*
*	LCD 2004 Test Routine / Fr Atmel Studio 7 und Atmel ATMEGA128A Core 7,3728 Mhz
*	Ellen Ramcke 10 2019

*	LCD Modul 2004A - V20 mit i2c PCF 8574T Adapter. Alle Jumper offen, Adresse 0x27 << 1 = 0x4E Schreiben 0x4F Lesen
*	Das Modul hat einen HD 44780 Controller. Gekauft bei Ebay aus China. Kosten 3,54
*
*	Grundlage war dieser Beitrag https://www.mikrocontroller.net/topic/334653 Autor Falk B
*	Ich habe die i2c Routinen komplett ausgetauscht gegen die von 
*	Author	: CC Charmane's, Chennai (India)
*	Diese laufen im mehreren Anwendungen bei mir stabil. 
*	Die Pinzuordnung entspricht den in diesem thread gezeigten Informationen
*	Die Pins habe ich gemessen zwischen  PCF und 16poligem header. Ist also kompatibel. 
*
*	Diese Einstellung funktionieren fr 20x4 Zeilen (in i2clcd.h):
*	#define LCD_LINE1			0x00	    < This should be 0x00 on all displays
*	#define LCD_LINE2			0x40	    < Change this to the address for line 2 on your display
*	#define LCD_LINE3			0x14	    < Change this to the address for line 3 on your display
*	#define LCD_LINE4			0x54	    < Change this to the address for line 4 on your display
*
*	Super Display. Viel Spass damit wnsche ich.
*
*	Danksagung noch an die Autoren der i2clcd Lib. Luft, Original belassen.
*	Copyright (C) 2006 Nico Eichelmann and Thomas Eichelmann
*	2014 clean up by Falk Brunner
*
*/




#include <stdbool.h>
#include <stdint.h> 
#include <avr/pgmspace.h>
#define F_CPU 7372800UL	// Hinweis: dies muss vor util/delay deklariert werden
#include <util/delay.h>
#include "i2clcd.h"
#include "i2c_routines.h"
#include "LED0_3.h"

char *msg[] = {"Hello World ....","Test LCD 2004","0123456789","abcdefghij"};

volatile uint8_t buf,cnt,err;



/** 
	Prfen ob das Gert bereit ist:
	ein SLW + ACK schreiben, Abfrage ACK, Gert bereit, return TWSR 
	0x18 ACK OK, 0x20 NACK slave nicht bereit 
	Siehe auch Datasheet ATmega128 Seite 214 */
uint8_t CeckDevicePresent()
{
	uint8_t res;
	I2C_Start();
	I2C_Write(LCD_I2C_Address);
	res = TWSR;
	I2C_Stop();
	return res;
}


int main(void)
{
	PortA_init;
	I2C_Init(); // 
	
	cnt = 0;
	// i2c Bus trace: T20191022
	// check device ready .. ACK on SLW received
	do
	{err = CeckDevicePresent(); cnt++;}
	//while (err != 0x20); //	check bis kein NAK mehr
	while (err != 0x18);   //	check bis salve mit ACK anwortet
	
	lcd_init();
	lcd_light(true);
	

    // always set all three parameters  (ON/OFF) when using this command
	//lcd_command(LCD_DISPLAYON | LCD_CURSORON | LCD_BLINKINGON);
 
    while (1) 
	{	
		
		_delay_ms(1000); cnt=0;
		lcd_command(LCD_CLEAR);
		LED0_ON;
		_delay_ms(300);
		LED0_OFF;
		lcd_printlc(cnt+1, 1 + 2*cnt , msg[cnt]); cnt++;
		_delay_ms(300);
		lcd_printlc(cnt+1, 1 + 2*cnt , msg[cnt]); cnt++;
		_delay_ms(300);
		lcd_printlc(cnt+1, 1 + 2*cnt , msg[cnt]); cnt++;		
		_delay_ms(300);
		lcd_printlc(cnt+1, 1 + 2*cnt , msg[cnt]); cnt++;		
		
    }
}
